//*****************************************************************************
//*
//*
//*		AzArchiverDoc.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2010
//
#include	"StdAfx.h"
#include	"AzArchiver.h"
#include	"AzArchiverDoc.h"
#include	"AzArchiverView.h"
#include	"PasswordDlg.h"
#include	"ProgressDlg.h"
#include	"ConfigDlg.h"


#ifdef		_DEBUG
#define new	 DEBUG_NEW
#undef		 THIS_FILE
static char	 THIS_FILE[] = __FILE__;
#endif
		    

IMPLEMENT_DYNCREATE(CAzArchiverDoc, CDocument)

BEGIN_MESSAGE_MAP(CAzArchiverDoc, CDocument)
	//{{AFX_MSG_MAP(CAzArchiverDoc)
	ON_COMMAND(ID_NEW_ZIP,	OnNewZip)
	ON_COMMAND(ID_COPYPATH,	OnCopyPath)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

#define 	ENTRIES(a)		(sizeof(a)/sizeof((a)[0]))


BYTE		CAzArchiverDoc::m_aCharWidths[256];
int		   *CAzArchiverDoc::m_pTextWidth = 0;
int 		CAzArchiverDoc::m_iCharInit  = 0;
CString		CAzArchiverDoc::m_sPassword;

//*****************************************************************************
//*
//*		CAzArchiverDoc
//*
//*****************************************************************************
// 	CAzArchiverDoc Konstruktion/Destruktion
CAzArchiverDoc::CAzArchiverDoc()
{
	m_pArchive		= NULL;
	m_bChanged		= FALSE;
	m_bNotOverWrite = FALSE;
}

//*****************************************************************************
//*
//*		CAzArchiverDoc
//*
//*****************************************************************************
CAzArchiverDoc::~CAzArchiverDoc()
{
	if(m_pArchive)
		{
		CloseArchiv();
		}
}

//*****************************************************************************
//*
//*		OnNewDocument
//*
//*****************************************************************************
BOOL CAzArchiverDoc::OnNewDocument()
{

	if(!CDocument::OnNewDocument())
		{
		return FALSE;
		}

	if(m_pArchive)
		{
		CloseArchiv();
		}

	m_bChanged = FALSE;

	UpdateAllViews(NULL,1);


return TRUE;
}

//*****************************************************************************
//*
//*		CloseArchiv
//*
//*****************************************************************************
void CAzArchiverDoc::CloseArchiv()
{
CAzArchiverView	*pView;
int				 iError;

	GET_VIEW(pView,CAzArchiverView);

	if(m_bChanged && pView && m_pArchive->HasCommpress())
		{
		CProgressDlg	sDlg(pView);

		InitProgress(&sDlg);

		sDlg.SetTitle(_T("Archiv erstellen"));
		sDlg.SetProc (CloseProc,m_pArchive);
		sDlg.SetTime (TimeProc,250);
		sDlg.DoModal ();

		ExitProgress();

		iError = sDlg.GetExitCode();
		}
	else{
		iError = m_pArchive->Close();
		}

	if(iError!=CAF_ERROR_OK)
		{
		CString		sText;
		CWnd	   *pWnd;

		sText.Format("Fehler (%i) beim Schlieen des Archives !\n%s",iError,m_pArchive->ErrorText());

			pWnd=AfxGetMainWnd();
		if(!pWnd)
			{
			MessageBox(0,sText,"Fehler",MB_OK|MB_ICONERROR);
			}
		else{
			pWnd->MessageBox(sText,"Fehler",MB_OK|MB_ICONERROR);
			}
		}
								    

	delete m_pArchive;
	m_pArchive = NULL;
	m_bChanged = FALSE;

}

//*****************************************************************************
//*
//*		Serialize
//*
//*****************************************************************************
// 	CAzArchiverDoc Serialisierung
void CAzArchiverDoc::Serialize(CArchive& ar)
{
	if(ar.IsStoring())
		{
		}
	else{
		}
}


#ifdef _DEBUG


//*****************************************************************************
//*
//*		AssertValid
//*
//*****************************************************************************
void CAzArchiverDoc::AssertValid() const
{
	CDocument::AssertValid();
}

//*****************************************************************************
//*
//*		Dump
//*
//*****************************************************************************
void CAzArchiverDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif


//*****************************************************************************
//*
//*		OnOpenDocument
//*
//*****************************************************************************
BOOL CAzArchiverDoc::OnOpenDocument(LPCTSTR pPathName)
{
FILE   *pFile;
CString	sText;
char	cPassword[64];
char	cBuffer[512+4];
__int64	iLen64;
int		iError;
int		iFlag;
int		iLen;
int		iRet;


	SetCursor(IDC_WAIT);

	if(m_pArchive)
		{
		CloseArchiv();
		}

		pFile = _tfopen(pPathName,_T("rb"));
	if(!pFile)
		{
		sText.Format(_T("Die Datei %s konnte nicht geffnet werden !"),pPathName);
		AfxGetMainWnd()->MessageBox(sText,_T("Fehler"),MB_OK|MB_ICONERROR);
		SetCursor(IDC_ARROW);
		return FALSE;
		}

	iFlag = 0;
	iLen  = fread(cBuffer,1,512,pFile);
	cBuffer[iLen] = 0;
	cPassword[0]  = 0;

	if(iLen>4 && cBuffer[0]=='P' && cBuffer[1]=='K')		// Archivformat feststellen
		{
		m_pArchive = new CZipFile; iFlag=CAF_FLAG_NOCHECK;
		}
	else if(iLen>4 && cBuffer[0]=='B' && cBuffer[1]=='Z')
		{
		m_pArchive = new CBz2File;
		}
	else if(iLen>=512 && !strcmp(cBuffer+0x101,"ustar  "))
		{
		m_pArchive = new CTarFile;
		}
	else if(iLen>=512 && !memcmp(cBuffer,"\xFD""7zXZ\0",6))
		{
		m_pArchive = new CXzsFile;
		}
	else if(iLen>4 && cBuffer[0]=='\x1F' && cBuffer[1]=='\x8B')
		{
		m_pArchive = new CTgzFile;
		}
	else if(iLen>4 && cBuffer[0]=='\x1F' && cBuffer[1]=='\x9D')
		{
		m_pArchive = new CLzwFile;
		}
	else if(iLen>4 && cBuffer[0]=='\xED' && cBuffer[1]=='\xAB' && cBuffer[2]=='\xEE' && cBuffer[3]=='\xDB')
		{
		m_pArchive = new CRpmFile;
		}
	else if(iLen>4 && cBuffer[0]=='M' && cBuffer[1]=='S' && cBuffer[2]=='C' && cBuffer[3]=='F')
		{
		m_pArchive = new CCabFile;
		}
	else if(iLen>4 && cBuffer[0]=='I' && cBuffer[1]=='T' && cBuffer[2]=='S' && cBuffer[3]=='F')
		{
		m_pArchive = new CChmFile;
		}
	else if(iLen>4 && cBuffer[0]=='R' && cBuffer[1]=='a' && cBuffer[2]=='r' && cBuffer[3]=='!')
		{
		m_pArchive = new CRarFile;
		}
	else if(iLen>4 && cBuffer[0]=='U' && cBuffer[1]=='n' && cBuffer[2]=='i' && cBuffer[3]=='q')
		{
		m_pArchive = new CRarFile;
		}
	else if(iLen>6 && cBuffer[0]=='7' && cBuffer[1]=='z' && cBuffer[2]=='\xBC' && cBuffer[3]=='\xAF' && cBuffer[4]=='\x27' && cBuffer[5]=='\x1C')
		{
		m_pArchive = new CZp7File;
		}
	else if(iLen>=512 && CheckTarHeader(cBuffer))
		{
		m_pArchive = new CTarFile;
		}
	else if(iLen>8 && cBuffer[0]=='\xD0' && cBuffer[1]=='\xCF' && cBuffer[2]=='\x11' && cBuffer[3]=='\xE0')
		{
		m_pArchive = new CMsiFile;
		}
	else if(iLen>6 && cBuffer[0]=='0' && cBuffer[1]=='7' && cBuffer[2]=='0' && cBuffer[3]=='7' && cBuffer[4]=='0' && cBuffer[5]>='1' && cBuffer[5]<='7')
		{
		m_pArchive = new CPioFile;
		}
	else if(iLen>6 && cBuffer[0]=='\xC7' && cBuffer[1]=='\x71')
		{
		m_pArchive = new CPioFile;
		}
	else if(iLen>6 && cBuffer[0]=='\x71' && cBuffer[1]=='\xC7')
		{
		m_pArchive = new CPioFile;
		}
	else if(iLen>=512 && cBuffer[0]=='M' && cBuffer[1]=='Z' && CheckHxsHeader(pFile,cBuffer))
		{
		m_pArchive = new CChmFile;
		}
	else if(iLen>=512 && cBuffer[0]=='M' && cBuffer[1]=='Z')
		{
		m_pArchive = new CExeFile;
		}
	else if(iLen>=512 && cBuffer[0]=='P' && cBuffer[1]=='E')
		{
		m_pArchive = new CExeFile;
		}
	else if(iLen>=512 && memcmp(cBuffer,"\x00\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x00",12)==0)
	 	{
		m_pArchive = new CIsoFile;
		}
	else if(iLen>=64 && CheckUueHeader(cBuffer,iLen))
		{
		m_pArchive = new CUueFile;
		}
	else if(iLen>=512 && CheckIsoHeader(pFile))
		{
		m_pArchive = new CIsoFile;
		}
	else if(iLen>=16 && CheckCueHeader(cBuffer,pPathName))
		{
		m_pArchive = new CIsoFile;
		}
	else if(iLen>5+8 && CheckLzmHeader(cBuffer))
		{
		m_pArchive = new CLzmFile;
		}
	else if(iLen>=512 && cBuffer[510]=='\x55' && cBuffer[511]=='\xAA')
		{
		m_pArchive = new CFatFile;
		}
	else{
		fclose(pFile);
		sText.Format(_T("Die Datei %s hat ein unbekanntes Format !"),pPathName);
		AfxGetMainWnd()->MessageBox(sText,_T("Fehler"),MB_OK|MB_ICONERROR);
		SetCursor(IDC_ARROW);
		return FALSE;
		}

	fclose(pFile);
	LengthFile(pPathName,&iLen64);

	if(!m_sPassword.IsEmpty())									// Passwort fr's ffnen
		{
		m_pArchive->SetPassword(m_sPassword);
		m_sPassword.Empty();
		}

Retry:	  

	if(iLen64<0x500000)											// Kleine Datei direkt ffnen
		{
		iError = m_pArchive->Open(pPathName,CAF_READ|iFlag);
		}
	else{														// Verlaufsdialog anzeigen
		CAzArchiverView	*pView;
		GET_VIEW(pView,CAzArchiverView);
		CProgressDlg	sDlg(pView);

		InitProgress(&sDlg);

		sDlg.HideCancel();
		sDlg.SetTitle(_T("Archiv ffnen"));
		sDlg.SetProc(OpenProc,m_pArchive,pPathName,CAF_READ);
		sDlg.SetTime(TimeProc,250);
		sDlg.DoModal();

		iError = sDlg.GetExitCode();

		ExitProgress();
		}


	if(iError==CAF_ERROR_PASSWD)									// Falsches Passwort
		{
		   iRet = AfxGetMainWnd()->MessageBox("Das Password ist ungltig !\nSoll ein anderes Password eingegben werden ?",_T("Fehler"),MB_YESNO|MB_ICONERROR);
		if(iRet==IDYES)
			{
			CPasswordDlg	sText;

			sText.SetRegPath(_T("PasswordDlg"));
			sText.SetText(cPassword);

			if(sText.DoModal()==IDOK)
				{
				sText.GetText(cPassword,sizeof(cPassword));
				m_pArchive->SetPassword(cPassword);
				goto Retry;
				}
			}

		delete m_pArchive;
		m_pArchive = NULL;
		SetCursor(IDC_ARROW);
		UpdateAllViews(NULL,2);
		return FALSE;
		}
	else if(iError!=CAF_ERROR_OK)									// Gab es einen Fehler
		{
		AfxGetMainWnd()->MessageBox(m_pArchive->ErrorText(),_T("Fehler"),MB_OK|MB_ICONERROR);
		delete m_pArchive;
		m_pArchive = NULL;
		SetCursor(IDC_ARROW);
		UpdateAllViews(NULL,2);
		return FALSE;
		}

	SetPathName(pPathName);
	AfxGetApp()->AddToRecentFileList(pPathName);


	UpdateAllViews(NULL,2);
	SetCursor(IDC_ARROW);
	SetTitle(GetFileName(pPathName));
	GetFullPathName(pPathName,ENTRIES(cBuffer),cBuffer,NULL);
	m_sArchivePath = cBuffer;



return TRUE;
}

//*****************************************************************************
//*
//*		OnCreateDocument
//*
//*****************************************************************************
//	Erzeugt ein neues Archiv
BOOL CAzArchiverDoc::OnCreateDocument(LPCTSTR pPathName)
{
TCHAR		cBuffer[512+4];
LPCTSTR		pType;
CString		sText;
int			iError;



	if(!m_bNotOverWrite && ExistFile(pPathName)==1)
		{
		sText.Format(_T("Die Datei '%s' existiert bereits.\nSoll sie berschreiben werden ?"),pPathName);

		if(AfxGetMainWnd()->MessageBox(sText,_T("Frage"),MB_YESNO|MB_ICONQUESTION)!=IDYES)
			{
			return FALSE;
			}
		}

	m_bNotOverWrite = 0;
	SetCursor(IDC_WAIT);

	if(m_pArchive)
		{
		CloseArchiv();
		}


	pType = GetFileName(pPathName);

	if(CmpFileType(pType,_T("bin")))						// "*.zip.bin" in "*.zip" umwandeln
		{
		GetFileBody(cBuffer,pType,512);
		pType = cBuffer;
		}


	if(CmpFileType(pType,_T("zip;jar;zipx")))
		{
		m_pArchive = new CZipFile;
		}
	else if(CmpFileType(pType,_T("bz2")))
		{
		m_pArchive = new CZipFile;
		}
	else if(CmpFileType(pType,_T("tar")))
		{
		m_pArchive = new CZipFile;
		}
	else if(CmpFileType(pType,_T("xz")))
		{
		m_pArchive = new CXzsFile;
		}
	else if(CmpFileType(pType,_T("gz;tgz")))
		{
		m_pArchive = new CTgzFile;
		}
	else if(CmpFileType(pType,_T("lzw")))
		{
		m_pArchive = new CLzwFile;
		}
	else if(CmpFileType(pType,_T("cab")))
		{
		m_pArchive = new CCabFile;
		}
	else if(CmpFileType(pType,_T("uue")))
		{
		m_pArchive = new CUueFile;
		}
	else if(CmpFileType(pType,_T("poio")))
		{
		m_pArchive = new CPioFile;
		}
	else if(CmpFileType(pType,_T("lzma")))
		{
		m_pArchive = new CLzmFile;
		}
	else{
		pType = CConfigDlg::GetCreateType();

		if(!_tcsicmp(pType,_T("bz2")))
			{
			m_pArchive = new CZipFile;
			}
		else if(!_tcsicmp(pType,_T("tar")))
			{
			m_pArchive = new CZipFile;
			}
		else if(!_tcsicmp(pType,_T("gz;tgz")))
			{
			m_pArchive = new CTgzFile;
			}
		else if(!_tcsicmp(pType,_T("cab")))
			{
			m_pArchive = new CCabFile;
			}
		else if(!_tcsicmp(pType,_T("uue")))
			{
			m_pArchive = new CUueFile;
			}
		else if(!_tcsicmp(pType,_T("poio")))
			{
			m_pArchive = new CPioFile;
			}
		else if(!_tcsicmp(pType,_T("lzma")))
			{
			m_pArchive = new CLzmFile;
			}
		else{
			m_pArchive = new CZipFile;
			}
		}


	   iError = m_pArchive->Open(pPathName,CAF_CREATE);
	if(iError!=CAF_ERROR_OK)
		{
		AfxGetMainWnd()->MessageBox(m_pArchive->ErrorText(),_T("Fehler"),MB_OK|MB_ICONERROR);
		delete m_pArchive;
		m_pArchive = NULL;
		SetCursor(IDC_ARROW);
		UpdateAllViews(NULL,2);
		return FALSE;
		}

	m_bChanged = 1;
	SetPathName(pPathName);
	AfxGetApp()->AddToRecentFileList(pPathName);


	UpdateAllViews(NULL,2);
	SetCursor(IDC_ARROW);
	SetTitle(GetFileName(pPathName));
	GetFullPathName(pPathName,sizeof(cBuffer)/sizeof(TCHAR),(LPTSTR)cBuffer,NULL);
	m_sArchivePath = (LPTSTR)cBuffer;


return TRUE;
}

//*****************************************************************************
//*
//*		OnSaveDocument
//*
//*****************************************************************************
BOOL CAzArchiverDoc::OnSaveDocument(LPCTSTR pPathName)
{
CAzArchiverView	   *pView;
int					iErr;


	if(!m_pArchive)return FALSE;



	if(m_pArchive->GetMode()==CAF_READ)
		{
		return TRUE;
		}


	if(CmpFileName(m_sArchivePath,pPathName))
		{
		return FALSE;
		}


	if(!m_pArchive->HasCommpress())
		{
		return FALSE;
		}



	GET_VIEW(pView,CAzArchiverView);
	CProgressDlg	sDlg(pView);


	InitProgress(&sDlg);

	sDlg.HideCancel();
	sDlg.SetTitle(_T("Archiv speichern"));
	sDlg.SetProc(ChangeProc,m_pArchive,CAF_READ);
	sDlg.SetTime(TimeProc,250);
	sDlg.DoModal();

	iErr = sDlg.GetExitCode();

	ExitProgress();

	if(iErr==CAF_ERROR_OK)
		{
		m_bChanged = 0;
		SetModifiedFlag(FALSE);
		}
	else{
		pView->MessageBox(m_pArchive->ErrorText(),_T("Fehler"),MB_OK|MB_ICONERROR);
		}




return TRUE;
}


//*****************************************************************************
//*
//*		OnCloseDocument
//*
//*****************************************************************************
void CAzArchiverDoc::OnCloseDocument()
{

	if(m_pArchive)
		{
		CloseArchiv();
		}

	CDocument::OnCloseDocument();

}


//*****************************************************************************
//*
//*		CloseProc
//*
//*****************************************************************************
//	Funktion fr das ffnen eines Archives
int _cdecl CAzArchiverDoc::CloseProc(CProgressDlg *pDlg)
{
CArchiveFile	   *pArchive=(CArchiveFile*)pDlg->GetVoid1();
int					iError;
__int64				iMax;



	iError  = pArchive->Close();

	pDlg->GetRange(iMax);
	pDlg->SetPos(iMax);




return iError;
}

//*****************************************************************************
//*
//*		OpenProc
//*
//*****************************************************************************
//	Funktion fr das ffnen eines Archives
int _cdecl CAzArchiverDoc::OpenProc(CProgressDlg *pDlg)
{
CArchiveFile   *pArchive  =(CArchiveFile*)pDlg->GetVoid1();
LPCTSTR			pPathName =               pDlg->GetChar2();
int				iMode     =				  pDlg->GetInt3 ();
int				iError;
__int64			iMax;



 	iError  = pArchive->Open(pPathName,iMode|CAF_FLAG_NOCHECK);

	pDlg->GetRange(iMax);
	pDlg->SetPos(iMax);




return iError;
}

//*****************************************************************************
//*
//*		ChangeProc
//*
//*****************************************************************************
//	Funktion fr das Speichern eines Archives
int _cdecl CAzArchiverDoc::ChangeProc(CProgressDlg *pDlg)
{
CArchiveFile   *pArchive  =(CArchiveFile*)pDlg->GetVoid1();
int				iMode     =				  pDlg->GetInt2 ();
int				iError;
__int64			iMax;



	iError  = pArchive->Change(iMode);

	pDlg->GetRange(iMax);
	pDlg->SetPos(iMax);




return iError;
}


//*****************************************************************************
//*
//*		TimeProc
//*
//*****************************************************************************
//	Funktion fr das ffnen eines Archives
void _cdecl CAzArchiverDoc::TimeProc(CProgressDlg *pDlg)
{
CArchiveFile   *pArchive =(CArchiveFile*)pDlg->GetVoid1();
__int64			iPos,iMax;



	iPos=pArchive->Done(&iMax);

	pDlg->SetRange(iMax);
	pDlg->SetPos(iPos);


}

//*****************************************************************************
//*
//*		TextProc
//*
//*****************************************************************************
//	Callback fr Textanzeige beim ffnen
int _cdecl CAzArchiverDoc::TextProc(void *pParam,int iMode,int iSub,CArchiveFile *pFile,LPTSTR pText)
{
CProgressDlg   *pDlg=(CProgressDlg*)pParam;
TCHAR			cSign;
int				iSize;
int				iAdd;
int				i,j;
int				k,l;




	if(!pText)return 0;

	iAdd   = strwlen(pText,0xE0);
	pText += iAdd;


	i = pFile->m_iProgPos-iAdd;							// Ist die Position des Dateinamens im Progress-Text
	j = pFile->m_iProgLen;								// Ist die Lnge des Dateinamens im Progress-Text
	k = pFile->m_iProgAdd;								// Ist die Lnge des Progress-Textes nach dem Dateinamen
	l = (iSub>=0)? 1:0;


	if(j>0)
		{
		if(l && j>=2 && pText[j+1]==':')l=0;

		cSign		= pText[i+j];
		pText[i+j]	= 0;
		iSize		= CommpressFileName(pText+i,pText+i,j+1,*m_pTextWidth-15,m_aCharWidths,(l)? 1:2,pText,(k>1)? pText+i+j+1:0,k-1);
		pText[i+j]	= cSign;

		if(iSize!=j)memmove(pText+i+iSize,pText+i+j,(k+1)*sizeof(TCHAR));
		}

	pDlg->SetText(pText);



return 0;
}

//*****************************************************************************
//*
//*		CheckUueHeader
//*
//*****************************************************************************
//	Prft den Header einer UUE-Datei
int CAzArchiverDoc::CheckUueHeader(char *pHeader,int iLen)
{
int		iPos;



	iPos = strlfind(pHeader,"begin",iLen-5);

	if(iPos<0)return 0;
	if(iPos>0 && pHeader[iPos-1]!='\n')return 0;

	pHeader += iPos+5;

		iPos = strwlen(pHeader,0xA0);
	if(!iPos || iPos>8)return 0;

	pHeader += iPos;

		iPos = strwlen(pHeader,0x00020000);
	if(!iPos || iPos>15)return 0;

	pHeader += iPos;

		iPos = strcwlen(pHeader,0x40);
	if(!iPos || iPos>255)return 0;

	pHeader += iPos;

		iPos = strwlen(pHeader,0x40);
	if(!iPos || iPos>2)return 0;

	if(pHeader[iPos]<' ' || pHeader[iPos]>0x60)return 0;



return 1;
}


//*****************************************************************************
//*
//*		CheckCueHeader
//*
//*****************************************************************************
//	Prft den Header einer CUE-Datei
int CAzArchiverDoc::CheckCueHeader(char *pHeader,LPCTSTR &pPathName)
{
static TCHAR	cPath[256];
char			cFile[256];
int				iLen;


	for(;;)
		{
		if(!*pHeader)return 0;

		pHeader += strwlen(pHeader,0xE0);

		if(!memcmp(pHeader,"FILE",4) && strwchar(pHeader[4],0xE0))
			{
			pHeader += 4;
			break;
			}

		pHeader += strcwlen(pHeader,0xE0);
		}

	strgetstr(cFile,pHeader,sizeof(cFile));
	if(!cFile[0])return 0;

	iLen = GetFilePath(cPath,pPathName,sizeof(cPath)/sizeof(TCHAR));
	strawcpy(cPath+iLen,cFile,sizeof(cPath)/sizeof(TCHAR)-iLen);
	if(ExistFile(cPath)!=1)return 0;

	pPathName = cPath;


return 1;
}


//*****************************************************************************
//*
//*		CheckLzmHeader
//*
//*****************************************************************************
//	Prft den Header einer LZMA-Datei
int CAzArchiverDoc::CheckLzmHeader(char *pHeader)
{
__int64			iLen;
int				iPos;

	   iLen = *(__int64*)(pHeader+5);
	if(iLen<-1)return 0;

	   iLen >>= 34;
	if(iLen && iLen!=-1)return 0;

	   iPos = bit_find_1(pHeader+1,32);
	if(iPos<2 || iPos>30)return 0;

	   iPos = bit_find_1(pHeader+1,32,iPos+1);
	if(iPos<32)return 0;


return 1;
}

//*****************************************************************************
//*
//*		CheckTarHeader
//*
//*****************************************************************************
//	Prft den Header einer TAR-Datei
int CAzArchiverDoc::CheckTarHeader(void *pHeader)
{
unsigned char  *pData=(unsigned char*)pHeader;
unsigned		uSum;
unsigned		uPos;



	uSum = 0;



	for(uPos=148;uPos<156;uPos++)
		{
		if(pData[uPos]!=' ')break;
		}

	for(;uPos<156;uPos++)
		{
		if(pData[uPos]<'0')break;
		if(pData[uPos]>'9')break;

		uSum *= 8;
		uSum += pData[uPos]-'0';
		}

	for(uPos=0;uPos<148;uPos++)
		{
		uSum -= pData[uPos];
		}

	for(uPos=156;uPos<512;uPos++)
		{
		uSum -= pData[uPos];
		}

	uSum -= ' '*8;


return (uSum)? 0:1;
}

//*****************************************************************************
//*
//*		CheckIsoHeader
//*
//*****************************************************************************
//	Prft den Header einer ISO-Datei
int CAzArchiverDoc::CheckIsoHeader(void *pFilePtr)
{
FILE	   *pFile=(FILE*)pFilePtr;
char		cData[16]={0,0,0,0};


	if(fseek (pFile,16*2048,SEEK_SET))return 0;
	if(fread (cData,sizeof(cData),1,pFile)!=1)return 0;
	if(memcmp(cData+1,"CD001",5)==0)return 1;
	if(memcmp(cData+1,"BEA"  ,3)==0)return 1;
	if(fseek (pFile,-12,SEEK_END) )return 0;
	if(fread (cData,12,1,pFile)!=1)return 0;
	if(memcmp(cData  ,"NER5",4)==0)return 1;
	if(memcmp(cData+4,"NERO",4)==0)return 1;


return 0;
}

//*****************************************************************************
//*
//*		CheckHxsHeader
//*
//*****************************************************************************
//	Prft den Header einer HXS-Datei
int CAzArchiverDoc::CheckHxsHeader(void *pFilePtr,char *pHeader)
{
FILE 	   *pFile=(FILE*)pFilePtr;
char		cBuffer[512+4];
unsigned	uOff;
unsigned	uPos;



	memcpy(cBuffer,pHeader,512);
	pHeader = cBuffer;

	fseek(pFile,512,SEEK_SET);

	for(uOff=0;uOff<0x8000;uOff+=504)
		{
		for(uPos=0;uPos<=504;uPos++)
			{
			if(!memcmp(pHeader+uPos,"ITOLITLS",8))return 1;
			}

		memcpy(pHeader,pHeader+504,8);
		if(fread(pHeader+8,504,1,pFile)!=1)return 0;
		}



return 0;
}

//*****************************************************************************
//*
//*		ExitProgress
//*
//*****************************************************************************
//	Den Progress-Callback lschen
void CAzArchiverDoc::ExitProgress()
{

	if(!m_pArchive)return;

	m_pArchive->Progress();
	m_pArchive->ProgressMode(0);
	m_pTextWidth = 0;

}

//*****************************************************************************
//*
//*		InitProgress
//*
//*****************************************************************************
//	Den Progress-Callback einrichten
void CAzArchiverDoc::InitProgress(CProgressDlg *pDlg,int iMode)
{

	if(!m_pArchive)return;

	if(!m_iCharInit)
		{
		m_iCharInit	= 1;
		GetCharWidths(m_aCharWidths);
		}

	m_pArchive->Progress(pDlg,TextProc);
	m_pArchive->ProgressMode(iMode);
	m_pTextWidth = pDlg->GetWidth();
}

//*****************************************************************************
//*
//*		SetModifiedFlag
//*
//*****************************************************************************
void CAzArchiverDoc::SetModifiedFlag(BOOL bModified)
{

	m_bChanged = bModified;

}

//*****************************************************************************
//*
//*		OnNewZip
//*
//*****************************************************************************
void CAzArchiverDoc::OnNewZip()
{


	CFileDialog sDlg(FALSE,NULL,_T(""),OFN_EXPLORER|OFN_ENABLESIZING|OFN_PATHMUSTEXIST|OFN_HIDEREADONLY,_T("Archive|*.zip;*.tar;*.gz;*.bz2;*.cab;*.uue;*.cpio;*.xz;*.lzma|Alle Dateien|*.*|"));

	sDlg.m_ofn.lpstrTitle = _T("Neues Archiv erstellen");

	if(sDlg.DoModal()!=IDOK)return;

	OnCreateDocument(sDlg.GetPathName());

}

//*****************************************************************************
//*
//*		SetPassword
//*
//*****************************************************************************
void CAzArchiverDoc::SetPassword(LPCTSTR pPasswort)
{
	m_sPassword = pPasswort;
}

//*****************************************************************************
//*
//*		OnCopyPath
//*
//*****************************************************************************
void CAzArchiverDoc::OnCopyPath()
{

	SetClipboardText(m_strPathName);

}
