//*****************************************************************************
//*
//*
//*		PictureBoxView.h
//*
//*
//*****************************************************************************
#ifndef		__PICTUREBOXVIEW_H__
#define     __PICTUREBOXVIEW_H__

#if _MSC_VER >= 1000
#pragma once
#endif

#include	"SemaphoreEx.h"
#include	"Image\BitmapImage.h"

#define 	CPB_CACHE_SIZE		32

#ifndef		ID_SHOWFULLHEIGHT
#define 	ID_SHOWFULLHEIGHT	0x4B01
#endif
#ifndef		ID_SHOWFULLWIDTH
#define 	ID_SHOWFULLWIDTH	0x4B02
#endif
#ifndef		ID_SHOWFULLSIZE
#define 	ID_SHOWFULLSIZE		0x4B03
#endif
#ifndef		ID_SHOWSMALLER
#define 	ID_SHOWSMALLER		0x4B04
#endif
#ifndef		ID_SHOWBIGGER
#define 	ID_SHOWBIGGER		0x4B05
#endif
#ifndef		ID_SHOWCOMMON
#define 	ID_SHOWCOMMON		0x4B06
#endif

class		CPictureBoxFrame;

//*****************************************************************************
//*
//*		Klasse CPictureBoxView
//*
//*****************************************************************************
class CPictureBoxView : public CView
{
protected:

	int				m_iShowFormat;					// Zoom Format (0=noral -1=volle Breite -2=volle Breite -3=voller Schirm)
	int				m_iLoadImg;						// Bild das geladen werden soll
	int				m_iLastError;					// Welcher Slot hatte zuletzt einen Fehler
	int				m_iLastErrCnt;					// Welcher Slot wurde zuletzt wie oft geladen
	int				m_iSwitchImg;					// Bild zu dem geschalten werden muss wenn geladen
	int				m_iStretchImg;					// Gezoomtes Bild aus dem Cache (-1=keines)
	int	volatile	m_iCurrendImg;					// Ist das Bild	aus dem Cache das angezeigt wird
	BitmapImage		m_sStretchImg;					// Gezoomtes angezeigtes Bild
	BitmapImage		m_aImages [CPB_CACHE_SIZE];		// Images fr den Cache
	BitmapImage	   *m_pImagePtr;					// Zeiger auf angezeigtes Bild (0=keines)
	BOOL			m_bZoomed;						// Ist das Fenster gezoomt
	BOOL			m_bMouseWheel;					// Modus fr das Mausrad (0=Scrollen 1=Zommen)
	BOOL			m_bScrollOnX;					// Ist der X-Scrollbalken sichtbar
	BOOL			m_bScrollOnY;					// Ist der Y-Scrollbalken sichtbar
	BOOL			m_bStretchOn;					// Ist das Autozoomen der Bilder ein

	int				m_iScrollX;						// Scroll-Position
	int				m_iScrollY;						//		*
	int				m_iScreenX;						// Fenstergre
	int				m_iScreenY;						//		*

	CBrush			m_sBgBrush;						// Hintergrundfarbe


	int				m_iImageSizeX;					// Bildgre in Pixel mit Zoom
	int				m_iImageSizeY;					//		*
	int				m_iImageStartX;					// Startposition des Bildes im Fenster
	int				m_iImageStartY;					//		*
	int				m_iImageStopX;					// Endposition des Bildes im Fenster
	int				m_iImageStopY;					//		*
	int				m_iImageEndX;					// Endposition des Bildes mit Rahmen
	int				m_iImageEndY;					//		*

	CRect			m_sTitleRect;					// Aktuelles Rechteck vom Titel
	BOOL			m_bTitleMode;					// Dateinamen als Namen oder Pfad
	BOOL			m_bTitlelOn;					// Bildnamen eingeblentet
	LPCTSTR			m_pTitle;						// Zeiger auf den Bildnamen
	int				m_iTitleX;						// Position des Bildnamens
	int				m_iTitleY;						//		*
	int				m_iTitleTime;					// Updatezeit fr Bildnamen

	CPoint			m_sMovePoint;					// Move Point
	int   			m_bMoveOn;						// Bild verschieben (0=aus 1=Left 2=Right 3=Mittle)
	int				m_iMoveX;						// Scroll-Position
	int				m_iMoveY;						//		*

	BYTE			m_aChars[256];					// Zeichenbreiten
	TCHAR			m_aImgPath[CPB_CACHE_SIZE*256];	// Dateinamen des Bildes
	TCHAR			m_aImgName[CPB_CACHE_SIZE*256];	// Bildnamen
	BYTE  volatile	m_aImgLock[CPB_CACHE_SIZE];		// Ist dieser Slot
	DWORD volatile	m_uImgLast[CPB_CACHE_SIZE];		// Zuletzt augeschalten
	CSemaphoreNull	m_sThreadSignal;				// Signal fr Thread

	BOOL volatile	m_bThreadOn;					// Ist ein Thread aktiv
	BOOL volatile	m_bThreadStop;					// Den Thread abbrechen



//*****************************************************************************
//*
//*		Funktionen
//*
//*****************************************************************************

public:

			 CPictureBoxView();
	virtual ~CPictureBoxView();

	DECLARE_DYNCREATE(CPictureBoxView)

	friend		CPictureBoxFrame;


	int			SetPicture(LPCTSTR pPath,LPCTSTR pTitle=NULL,BOOL bCheckSufix=TRUE);
	BOOL		SetShowSize(int iMode=-6,BOOL bRedraw=TRUE,BOOL bLock=TRUE);
	BOOL		SetZoomSize(int iMode   ,BOOL bMiddle=TRUE);
	void		OnShowMode (int iSize);
	int 		WheelMode(int iMode=-1);
	int			TitleMode(int iMode=-1);
	void		ClearCache();

protected:


	#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
	#endif

	void		CheckScroll();
	void		FreePictures(int iCount=1,int iIgnore=-1);
	int			Load(int iSlot,BOOL bAllowDummy=0);
	static unsigned int ThreadProc(CPictureBoxView *pThis);


	//{{AFX_VIRTUAL(CPictureBoxView)
	public:
	virtual void OnDraw(CDC *pDC);
	virtual BOOL PreCreateWindow(CREATESTRUCT& sCreate);
	virtual void OnUpdate(CView *pSender,LPARAM lHint,CObject* pHint);
protected:
	//}}AFX_VIRTUAL



	//{{AFX_MSG(CPictureBoxView)
	afx_msg void OnZoom();
	afx_msg void OnCopy();
	afx_msg void OnCheck();
	afx_msg void OnSwitch();
	afx_msg void OnRedraw();
	afx_msg void OnDestroy();

	afx_msg void OnShowCommon();
	afx_msg void OnShowBigger();
	afx_msg void OnShowSmaller();
	afx_msg void OnShowFullSize();
	afx_msg void OnShowFullWidth();
	afx_msg void OnShowFullHeight();
	afx_msg void OnSelNamePath();
	afx_msg void OnWheelMode();

	afx_msg int  OnCreate		  (LPCREATESTRUCT pCreateStruct);
	afx_msg void OnKeyDown		  (UINT nChar,UINT nRepCnt,UINT nFlags);
	afx_msg void OnHScroll		  (UINT uCode,UINT uPos,CScrollBar *pScrollBar);
	afx_msg void OnVScroll		  (UINT uCode,UINT uPos,CScrollBar *pScrollBar);
	afx_msg void OnSize   		  (UINT uType,int iSizeX,int iSizeY);
	afx_msg void OnTimer  		  (UINT_PTR uIdEvent);

	afx_msg BOOL OnMouseWheel 	  (UINT nFlags,short  sDelta,CPoint sPoint);
	afx_msg void OnMouseMove  	  (UINT nFlags,CPoint sPoint);
	afx_msg void OnRButtonDown	  (UINT nFlags,CPoint sPoint);
	afx_msg void OnMButtonDown	  (UINT nFlags,CPoint sPoint);
	afx_msg void OnLButtonDown	  (UINT nFlags,CPoint sPoint);
	afx_msg void OnRButtonUp  	  (UINT nFlags,CPoint sPoint);
	afx_msg void OnMButtonUp  	  (UINT nFlags,CPoint sPoint);
	afx_msg void OnLButtonUp  	  (UINT nFlags,CPoint sPoint);

	afx_msg void OnMenuSelNamePath(CCmdUI *pCmdUI);
	afx_msg void OnMenuWheelMode  (CCmdUI *pCmdUI);
	afx_msg void OnMenuSmaller    (CCmdUI *pCmdUI);
	afx_msg void OnMenuBigger     (CCmdUI *pCmdUI);
	afx_msg void OnMenuImage	  (CCmdUI *pCmdUI);
	afx_msg void OnMenuSize       (CCmdUI *pCmdUI);

	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};


//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************
inline int CPictureBoxView::WheelMode(int iMode){if(iMode>=0)m_bMouseWheel=iMode;return m_bMouseWheel;}
inline int CPictureBoxView::TitleMode(int iMode){if(iMode>=0)m_bTitleMode =iMode;return m_bTitleMode ;}



//{{AFX_INSERT_LOCATION}}

#endif
